clear, close all, clc;
% Programmer   : Muhammad Agus Zainuddin
% Last editted : 9 March 2016
% Description  : Low Weight Coding for image transmission

%-----------   Input   ----------------
% lena256.bmp
% cancer256.bmp
% blood1.tif

%-----------  Input  ----------------
I = double(imread('cancer128.bmp'));          % Read the Image
data = reshape(I',1,size(I,1)*size(I,2));   % convert to single stream symbols
data_bit = de2bi(data,8,'left-msb');        % convert each symbol to binary
clear data;
binary_seq = reshape(data_bit',1,size(data_bit,1)*size(data_bit,2)); 
L = length(binary_seq)
clear data_bit;
num_1_uncoded = sum(binary_seq)                       % Number of 1 in binary input stream
%Percent_uncode = 100 * num_1_uncoded / length(binary_seq);   % Percentage of 1 in binary input stream
txrx = L + num_1_uncoded
% --- LWC encoding n-bits ---
k = 7; % 
n = 16;
load lwc_weight_3.mat;  
comb = lwc_3(1:2^k); clear lwc_3;

% Coding table
tab = zeros(2^k,2);
for i = 1:2^k, 
    tab(i,:) = [i-1 comb(i)];
end
LWC_sym = de2bi(comb,n,'left-msb'); 

disp('--- Encoding Process ---'),
binary_nbits = buffer(binary_seq,k)';
%clear binary_seq;
symbol_nbits = bi2de(binary_nbits,'left-msb')';
clear binary_nbits;
trans_bits = zeros(length(symbol_nbits),n);
for i = 1:length(symbol_nbits),
    ind = find(symbol_nbits(i) == tab(:,1));    %ind2 indeks coding
    trans_bits(i,:) = LWC_sym(ind,:);
end
num_1_lwc = sum(sum(trans_bits))
Energy_efficiency = 100 * (num_1_uncoded - num_1_lwc) / num_1_uncoded
% Count the number of sequential 1
trans_bits_seq = reshape(trans_bits',1,size(trans_bits,1)*size(trans_bits,2));
num_bits_lwc = length(trans_bits_seq)
clear trans_bits;

txrx_lwc = num_bits_lwc + num_1_lwc

energy_per_bit = txrx_lwc / txrx
% Transmission channel
load noise_10cm.mat;
noise = noise(1:length(trans_bits_seq));
recvword = mod(trans_bits_seq + noise,2);
%recvword = trans_bits_seq; %cek if there is no errors.

%disp('--- Decoding Process ---');
%--- Estimate the transmitted symbols ---
symbol_recv = bi2de(buffer(recvword, n)','left-msb')';
clear recvword;
symbol_est = zeros(size(symbol_recv));
for j = 1:length(symbol_recv),
    for x = 1:size(tab,1),
        if (symbol_recv(j) == tab(x,2)),
            symbol_est(j) = tab(x,1);
            break;
        else
            symbol_est(j) = 0;
        end
    end
end  
clear symbol_recv;
sym_bit = de2bi(symbol_est,k,'left-msb'); clear symbol_est;
recv_bit = reshape(sym_bit',1,size(sym_bit,1)*size(sym_bit,2));
data_est = recv_bit(1:L); clear recv_bit;
[ner, ber_lwc] = biterr(binary_seq,data_est)

img_sym = bi2de(buffer(data_est,8)','left-msb');
%clear data_est;
Image_est = buffer(img_sym,size(I,1))'; 
clear img_sym;
Image_est = Image_est(1:size(I,1),:);
Q = 255;
MSE  = sum(sum((Image_est - I).^ 2)) / (size(I,1)*size(I,2));

Image = uint8(Image_est);
PSNRbits = 10*log10(Q*Q/MSE);

imshow(Image), title('LWC code'),
   xlabel(['PSNR = ',num2str(PSNRbits),' dB']) 