clear, close, clc;
% Programmer   : Muhammad Agus Zainuddin
% Last editted : 8 March 2016
% Minimum Energy Channel(MEC) Coding


%-----------   Input   ----------------
load chan_mec_3.mat; % load Distance: d, Bit error rate: ber0, ber1 and ber.
%load chan_uncoded.mat;
%load data_1Mb.mat; % For energy computation
% Binary stream

L = 1e6;
data_bit = randsrc(1, L,[0 1;.5 .5]);  % Information bits
Num_1_Uncode = sum(data_bit);
% MEC Coding.
%disp('--- Encoding Process ---');
din = 4;    % Minimum distance
k = 3;      % Input size
M = 2^k;    % The number of symbols
binary_nbits = buffer(data_bit,k)';  % convert binary stream to n bits symbol.
% clear binary_sequence;
symbols = bi2de(binary_nbits,'left-msb')';
clear binary_nbits;

% Statistics and codeword
prob = zeros(1,M);
for i = 1:M,
    prob(i) = length(find(symbols == (i-1))) / length(symbols);
end 
pmak = max(prob);
nmin = din + (M-2) * (din/2);
codeword = zeros(2^k,nmin);
if pmak < 0.5,
    codeword(1,1:2) = [1 1];
    for i = 2:M,
        temp = bi2de(codeword(i-1,:),'left-msb') / 4;
        codeword(i,:) = de2bi(temp,nmin,'left-msb');
    end
else
    codeword(2,1:4) = [1 1 1 1];
    for i = 3:M,
        temp = bi2de(codeword(i-1,:),'left-msb') / 4;
        codeword(i,:) = de2bi(temp,nmin,'left-msb');
    end
end
% % codeword symbol
codeword_sym = bi2de(codeword,'left-msb');  % Needed in decoding process

% Transmission
trans_bits = zeros(length(symbols),nmin);
for i = 1:length(symbols),
    ind = symbols(i);    %ind2 indeks coding
    trans_bits(i,:) = codeword(ind + 1,:);
end
size_trans = size(trans_bits,1);
% Performance evalutions
num_1_mec = sum(sum(trans_bits))
transmitted_length = size(trans_bits,1) * size(trans_bits,2)
Energy_efficiency = 100 * (Num_1_Uncode - num_1_mec) / Num_1_Uncode
Bandwidth_expansion = transmitted_length / L;
Probability_bit_1 = num_1_mec / transmitted_length; % Probability bit 0

trans_bits_seq = reshape(trans_bits',1,size(trans_bits,1)*size(trans_bits,2));
clear trans_bits;
ber_mec = zeros(size(d));
for i = 1:length(d),
    % Decoding Process
    disp(i)
    noise = zeros(size(trans_bits_seq));
    for j = 1:length(trans_bits_seq),
        acak = rand;
        if trans_bits_seq(j) == 0,
            if acak < ber0(i), % here
                noise(j) = 1;
            end
        elseif trans_bits_seq(j) == 1,
            if acak < ber1(i),    % here
                noise(j) = 1;
            end
        end
    end
    recvword = mod(trans_bits_seq + noise,2);
    % recvword = trans_bits_seq; %cek if there is no errors.
    %clear trans_bits_seq;
    recvword = buffer(recvword,nmin)';
    % Find the codeword estimation
    codeword_est = zeros(size(recvword));    
    for j = 1:size_trans,
        tmp = nmin; % Initial distance as output size;
        for kk = 1:M,   % cek for all possible symbols
            cek = sum(mod(recvword(j,:) + codeword(kk,:),2)); % count the distance
            if cek < tmp, % If the distance is small than before, set as estimated codeword
                tmp = cek;
                codeword_est(j,:) = codeword(kk,:);
            end
        end
    end
    
    % Symbol detection
    binary_nbits_est = zeros(size(codeword_est,1),k);
    for j = 1:size(codeword_est,1),
        ind = find(bi2de(codeword_est(j,:),'left-msb') == codeword_sym);
        binary_nbits_est (j,:) = de2bi(ind-1,k,'left-msb');
    end
    clear codeword_est; 
    % binary_nbits_est = de2bi(symbol_est,k,'left-msb'); 
%     clear symbol_est;
    binary_sequence_est = reshape(binary_nbits_est',1, ...
    size(binary_nbits_est,1) * size(binary_nbits_est,2));
    clear binary_nbits_est;
    data_sequence_est = binary_sequence_est(1:L);
    clear binary_sequence_est;
    [ner, ber_mec(i)] = biterr(data_sequence_est, data_bit);
end

% Theoretical
% P1 = sum(trans_bits_seq)/length(trans_bits_seq);
% P0 = 1 - P1;
% BER = ((ber1*P1) + (ber0 * P0))/log2(k);
ber_theo = zeros(1,length(d));
for i = 1:length(d),
    cek = 0;
    for j = 2:nmin,
        tmp = faktor(nmin,j) * (ber(i)^j) * ((1 - ber(i))^(nmin - j));
        cek = cek + tmp;
    end
    ber_theo(i) = cek;
end

loglog(d,ber,d,ber_mec,'-*',d,ber_theo,'-o','linewidth',2), % ,d,ber_mec,'-o'
xlabel('Distance (m)'), ylabel('Bit error rate'),
ylim([1/L 1]),
legend('Uncoded','MEC','Theoretical'), grid on % ,'Simulation',
