clear, close all, clc;
% Programmer   : Muhammad Agus Zainuddin
% Last editted : 5 April 2015
% Description  : Minimum Energy Channel (MEC) Coding for image 


%-----------   Input   ----------------
% lena256.bmp
% cancer256.bmp
% blood1.tif

%-----------  Input  ----------------
I = double(imread('cancer128.bmp'));        % Read the Image
data = reshape(I',1,size(I,1)*size(I,2));   % convert to single stream symbols
data_bit = de2bi(data,8,'left-msb');        % convert each symbol to binary
clear data;
binary_sequence = reshape(data_bit',1,size(data_bit,1)*size(data_bit,2));
L = length(binary_sequence);
clear data_bit;
num_1_uncoded = sum(binary_sequence);           % Number of 1 in binary input stream
% Percentage of 1 in binary input stream
% Percent_uncode = 100 * num_1_uncoded / length(binary_sequence);   
%txrx = L + num_1_uncoded

% --- MEC Encoding ---
din = 4;    % Minimum distance
k = 7;      % Input size
M = 2^k;    % The number of symbols
binary_nbits = buffer(binary_sequence,k)';  % convert binary stream to n bits symbol.
% clear binary_sequence;
symbols = bi2de(binary_nbits,'left-msb')';
clear binary_nbits;

% Statistics and codeword
prob = zeros(1,M);
for i = 1:M,
    prob(i) = length(find(symbols == (i-1))) / length(symbols);
end 
pmak = max(prob);
nmin = din + (M-2) * (din/2);
codeword = zeros(2^k,nmin);
if pmak < 0.5,
    codeword(1,1:2) = [1 1];
    for i = 2:M,
        temp = bi2de(codeword(i-1,:),'left-msb') / 4;
        codeword(i,:) = de2bi(temp,nmin,'left-msb');
    end
else
    codeword(2,1:4) = [1 1 1 1];
    for i = 3:M,
        temp = bi2de(codeword(i-1,:),'left-msb') / 4;
        codeword(i,:) = de2bi(temp,nmin,'left-msb');
    end
end
% % codeword symbol
codeword_sym = bi2de(codeword,'left-msb');  % Needed in decoding process

% Transmission
trans_bits = zeros(length(symbols),nmin);
for i = 1:length(symbols),
    ind = symbols(i);    %ind2 indeks coding
    trans_bits(i,:) = codeword(ind + 1,:);
end

trans_bits_seq = reshape(trans_bits',1,size(trans_bits,1)*size(trans_bits,2));
num_1_mec = sum(trans_bits_seq)
num_bits_mec = length(trans_bits_seq)
energy_efficiency = 100 * (num_1_uncoded - num_1_mec)/num_1_uncoded

txrx_mec = num_bits_mec + num_1_mec
clear trans_bits;

%energy_per_bit = txrx_mec / txrx
% Transmission channel
load noise_10cm.mat;
noise = noise(1:length(trans_bits_seq));
recvword = mod(trans_bits_seq + noise,2);
clear noise;
%recvword = trans_bits_seq; %cek if there is no errors.
%clear trans_bits_seq;
recvword = buffer(recvword,nmin)';
disp('Find the codeword estimation')
tic
codeword_est = zeros(size(recvword));    
for j = 1:size(codeword_est,1),
    tmp = nmin; % Initial distance as output size;
    for kk = 1:M,   % cek for all possible symbols
        cek = sum(mod(recvword(j,:) + codeword(kk,:),2)); % count the distance
        if cek < tmp, % If the distance is small than before, set as estimated codeword
            tmp = cek;
            codeword_est(j,:) = codeword(kk,:);
        end
    end
end
toc    
% Symbol detection
binary_nbits_est = zeros(size(codeword_est,1),k);
for j = 1:size(codeword_est,1),
    ind = find(bi2de(codeword_est(j,:),'left-msb') == codeword_sym);
    binary_nbits_est (j,:) = de2bi(ind-1,k,'left-msb');
end
clear codeword_est; 
% binary_nbits_est = de2bi(symbol_est,k,'left-msb'); 
clear symbol_est;
binary_sequence_est = reshape(binary_nbits_est',1, ...
size(binary_nbits_est,1) * size(binary_nbits_est,2));
clear binary_nbits_est;
data_sequence_est = binary_sequence_est(1:L);
clear binary_sequence_est;
[ner, ber_mec] = biterr(data_sequence_est, binary_sequence);

img_sym = bi2de(buffer(data_sequence_est,8)','left-msb');
clear data_sequence_est;
Image_est = buffer(img_sym,size(I,1))'; 
clear img_sym;
Image_est = Image_est(1:size(I,1),:);
Q = 255;
MSE  = sum(sum((Image_est - I).^ 2)) / (size(I,1)*size(I,2));

Image = uint8(Image_est);
PSNRbits = 10*log10(Q*Q/MSE);

imshow(Image), title('MEC code'),
   xlabel(['PSNR = ',num2str(PSNRbits),' dB']) 