clear, clc, close all;

% Simple block nanocode (SBN)
% 20 March 2016

% load data_1Mb.mat;
L = 1e6;    % Data length
k = 3;      % Message length
n = 6;      % Codeword length

data_bit = randsrc(1, L,[0 1;.5 .5]);  % Information bits

% NME encoding
n_nme = k; % NME 8 bits
binary_nBits = buffer(data_bit,n_nme)';
symbols = bi2de(binary_nBits,'left-msb')';  % Convert to transmitted symbols
clear binary_nBits;
simbol = unique(symbols);
% Statistic of symbols
sta = zeros(size(simbol));
for i = 1:length(simbol),
    sta(i) =  length(find(symbols==simbol(i)));
end
freq = zeros(length(simbol),2);
for i = 1:length(simbol),
    freq(i,:) = [simbol(i) sta(i)];
end
% clear simbol;
clear sta;
% Ordered symbols by its occurance
occur = zeros(size(freq));
for j = 1:size(freq,1) - 1, 
    [t1,t2] = max(freq(:,2));   % t2 = row of max occurance
    occur(j,:) = freq(t2,:);
    freq(t2,:) = [];
end
occur(end,:) = freq(end,:); clear freq;

% Mapping in NME
if k == 3,
    load comb_nme_3.mat; nme = nme_3; clear nme_3;
elseif k == 5,
    load comb_nme_5.mat; nme = nme_5; clear nme_5;
elseif k == 7,
    load comb_nme_7.mat; nme = nme_7; clear nme_7;
end

% NME mapping
M_sym = nme(1:length(simbol)); % clear N_sym;
NME_sym = de2bi(M_sym,n_nme,'left-msb');
% Num_1_Dictionary = sum(sum(NME_sym))  % Mistake before (Fatal mistake)
    
% disp('Mapping Process'),
NME_bits = zeros(length(symbols),n_nme);
for i = 1:length(symbols),
    ind = find(symbols(i) == occur(:,1));    %ind2 indeks coding
    NME_bits(i,:) = NME_sym(ind,:);
end
NME_sequence = reshape(NME_bits',1,size(NME_bits,1)*size(NME_bits,2));
clear NME_bits;

% ----- Channel coding -------
% Instead of create new matrix, just call from previous generated matrix.
load block_6_3;    % load the parity matrix and syndrome table.

% Counting the error correction capability
n0 = 1; n1 = 0; n2 = 0; n3 = 0; 
n4 = 0; n5 = 0; n6 = 0; n7 = 0; n8 = 0;
for i = 1:size(trt,1),
    cek = sum(trt(i,:));
    if cek == 1,
        n1 = n1 + 1;
    elseif cek == 2,
        n2 = n2 + 1;
    elseif cek == 3,
        n3 = n3 + 1;
    elseif cek == 4,
        n4 = n4 + 1;
    elseif cek == 5,
        n5 = n5 + 1;
    elseif cek == 6,
        n6 = n6 + 1;    
    elseif cek == 7,
        n7 = n7 + 1;    
    elseif cek == 8,
        n8 = n8 + 1;    
    end
end
nx = [n0 n1 n2 n3 n4 n5 n6 n7 n8];

% Transmission at TeraHertz band
load chan_sbn.mat; % load Distance: d, Bit error rate: ber0, ber1 and ber.
ber_theo = zeros(1,length(d));
for i = 1:length(d),
    prob_correct = 0;
    for j = 1:length(nx),
        A = ber(i)^(j-1);
        B = (1-ber(i))^(n-j+1);
        prob_correct = prob_correct + (nx(j) * A * B);
    end
    ber_theo(i) = 1 - prob_correct;
end

% Matrix generator
G = gen2par(H); % The same as G = [parity' eye(k)];

codeword = encode(NME_sequence,n,k,'linear',G); % clear trans_bits
number_transmit_bits = size(codeword,1) * size(codeword,2);
prob_1 = sum(sum(codeword)) / number_transmit_bits;

% SBN transmission
ber_sbn = zeros(size(d));
for i = 1:length(d);
    disp(i),
    noise = zeros(size(codeword));
    for j = 1:length(codeword),
        acak = rand;
        if codeword(j) == 0,
            if acak < ber0(i), % here
                noise(j) = 1;
            end
        elseif codeword(j) == 1,
            if acak < ber1(i),    % here
                noise(j) = 1;
            end
        end
    end
    recvword = mod(codeword + noise,2);
    clear noise;
%     recvword = codeword;
    % --- Simple block decoding ---
    x_est = decode(recvword,n,k,'linear',G,trt);
    % --- NME decoding ---
    NME_sym_est = bi2de(buffer(x_est,n_nme)','left-msb')';
    %clear sequence_est;
    symbols_est = zeros(size(NME_sym_est));
    for j = 1:length(NME_sym_est),
        ind = find(NME_sym_est(j) == nme);
        if ind > length(simbol),
            symbols_est(j) = 0;
        else
            symbols_est(j) = occur(ind,1);
        end
    end   
    binary_nbits_est = de2bi(symbols_est,n_nme,'left-msb');
    clear symbols_est;
    data_est = reshape(binary_nbits_est',1,size(binary_nbits_est,1)*size(binary_nbits_est,2));
    data_est = data_est(1:L);
    %     x_est = x_est(1:L);
    [ner, ber_sbn(i)] = biterr(data_bit, data_est);
end

loglog(d,ber,'-ro',d,ber_theo,'-gs',d,ber_sbn,'-b*','linewidth',2), % d,ber,'o',d,ber0,d,ber1,d,ber_block
xlabel('Distance (m)'), ylabel('Bit error rate'),
%ylim([1/L 1]),
ylim([1e-6 1]),
legend('Uncoded','Theo','SBN'), grid on

