clear, clc, close  all;

% Programmer   : Muhammad Agus Zainuddin
% Last editted : 21 March 2016
% Description  : Image transmission using SBN in nanocommunication. 


%-----------   Input   ----------------
% lena256.bmp
% cancer256.bmp
% blood1.tif
% cancer128.bmp
%-----------  Input  ----------------
I = double(imread('cancer128.bmp'));        % Read the Image
data = reshape(I',1,size(I,1)*size(I,2));   % convert to single stream symbols
data_bit = de2bi(data,8,'left-msb');        % convert each symbol to binary
clear data;
binary_sequence = reshape(data_bit',1,size(data_bit,1)*size(data_bit,2));
L = length(binary_sequence);
%clear data_bit;
num_1_uncode = sum(binary_sequence);         % Number of 1 in binary input stream

%txrx = L + num_1_uncode;
% Percentage of 1 in binary input stream
%Percent_uncode = 100 * num_1_uncode / length(binary_sequence);

% NME encoding
k = 3; % Input size
n_nme = k; % NME 8 bits
binary_nBits = buffer(binary_sequence,n_nme)';
symbols = bi2de(binary_nBits,'left-msb')';  % Convert to transmitted symbols
clear binary_nBits;
simbol = unique(symbols);
% Statistic of symbols
sta = zeros(size(simbol));
for i = 1:length(simbol),
    sta(i) =  length(find(symbols==simbol(i)));
end
freq = zeros(length(simbol),2);
for i = 1:length(simbol),
    freq(i,:) = [simbol(i) sta(i)];
end
% clear simbol;
% clear Dict;
clear sta;
% Ordered symbols by its occurance
occur = zeros(size(freq));
for j = 1:size(freq,1) - 1, 
    [t1,t2] = max(freq(:,2));   % t2 = row of max occurance
    occur(j,:) = freq(t2,:);
    freq(t2,:) = [];
end
occur(end,:) = freq(end,:); clear freq;
% Mapping in NME
if k == 3,
    load comb_nme_3.mat; nme = nme_3; clear nme_3;
elseif k == 5,
    load comb_nme_5.mat; nme = nme_5; clear nme_5;
elseif k == 7,
    load comb_nme_7.mat; nme = nme_7; clear nme_7;
end

M_sym = nme(1:length(simbol)); % clear N_sym;
NME_sym = de2bi(M_sym,n_nme,'left-msb');
% Num_1_Dictionary = sum(sum(NME_sym))  % Mistake before (Fatal mistake)
    
% disp('Encoding Process'),
nme_bits = zeros(length(symbols),n_nme);
for i = 1:length(symbols),
    ind = find(symbols(i) == occur(:,1));    %ind2 indeks coding
    nme_bits(i,:) = NME_sym(ind,:);
end
nme_sequence = reshape(nme_bits',1,size(nme_bits,1)*size(nme_bits,2));
clear trans_bits;

% --- Block encoding ---
n = 16; % Codeword length
load block_16_3;    % Load matrix G, H, and error pattern table
G = gen2par(H); % The same as G = [parity' eye(k)];
codeword_block = encode(binary_sequence,n,k,'linear',G); %clear trans_bits
num_1_block = sum(codeword_block)
clear codeword_block;
codeword_sbn = encode(nme_sequence,n,k,'linear',G); %clear trans_bits
num_1_sbn = sum(codeword_sbn)
num_bits_sbn = length(codeword_sbn)
%prob_1_sbn = num_1_sbn / num_bits_sbn
txrx_block = num_bits_sbn + num_1_block
txrx_sbn = num_bits_sbn + num_1_sbn
% energy_per_bit = txrx_sbn / txrx


% Energy efficiency sbn from block
Energy_Eff_sbn = 100 * (num_1_block - num_1_sbn) / num_1_block

% --- ChannEL properties ---
load noise_10cm.mat;
noise = noise(1:length(codeword_sbn));
recvword = mod(codeword_sbn + noise,2);
%recvword = codeword_sbn; % check only
clear codeword; clear noise;
sequence_est = decode(recvword,n,k,'linear',G,trt);
clear recvword;

% --- NME decoding ---
NME_sym_est = bi2de(buffer(sequence_est,n_nme)','left-msb')';
%clear sequence_est;
symbols_est = zeros(size(NME_sym_est));
for i = 1:length(NME_sym_est),
    ind = find(NME_sym_est(i) == nme);
    if ind > length(simbol),
        symbols_est(i) = 0;
    else
        symbols_est(i) = occur(ind,1);
    end
end
clear NME_sym_est;
binary_nbits_est = de2bi(symbols_est,n_nme,'left-msb');

data_est = reshape(binary_nbits_est',1,size(binary_nbits_est,1)*size(binary_nbits_est,2));
clear binary_nbits_est; 
data_est = data_est(1:L);
[ner, ber_sbn] = biterr(binary_sequence, data_est);
img_sym = bi2de(buffer(data_est,8)','left-msb');
%clear data_est;
Image_est = buffer(img_sym,size(I,1))'; 
clear img_sym;
Image_est = Image_est(1:size(I,1),:);
Q = 255;
MSE  = sum(sum((Image_est - I).^ 2)) / (size(I,1)*size(I,2));

Image = uint8(Image_est);
PSNRbits = 10*log10(Q*Q/MSE);

SSIM_sbn = ssim(uint8(I),Image);

imshow(Image), title('Block code'),
   xlabel(['PSNR = ',num2str(PSNRbits),' dB']) 
