% % Script to compute the capacity under different channel conditions
% Muhammad Agus Zainuddin
clear all
close all
clc

%% Initialization
% Time/Frequency
fs = 1e14; %sample rate-10 times the highest frequency
ts = 1/fs; %sample period
t = (0:1:2^16)*ts;
To = t(end) - t(1); % Inter-symbol periode

tmax = max(t); %for normalization purposes
% Beta = 1000; %pulse spreading

L = length(t);
NFFT = 2^nextpow2(L);
f = fs/2*linspace(0,1,NFFT/2+1);

% [max_f,cut_max] = max(f.*(f<10e12));
% [min_f,cut_min] = max(f.*(f<100e9));

% Pulse parameters
sigma = 100e-15;
t0 = 500e-15;

% HERE IS WHERE YOU ADJUST THE TRANSMITTED PULSE POWER. CHECK ALSO THE
% REVISED VERSION OF THE TSOOK PAPER FOR REASONABLE NUMBERS!
Emax = 1e-18;
Nderiv = 1;

% Medium parameters
Tstp = 273.15; %[K]
T0 = 296; %[K]
% P0 = 1; %[atm]
k = 1.380658e-23; %[J/K]
h = 6.62618e-34; %[J s]
c = 2.997925e8; %[m/s]
R = 82.0575; %[cm^3 atm k^-1 mol^-1]
NA = 6.022141e23; %[mol^-1]

% System Conditions
P = 1; %[atm]
T = 296; %[K]

% Medium Composition
water = 100000;
bands = [2011,331,9720,446,102440,32064,500,4765];
con = [209460,383,1.745,0.3,0.05,0.02,0.1,water];
mass = [32,44.01,16.04,44.01,48,46.01,28.01,18.02]./NA;

%%%%%%%%%%
% distances
lend = 20; %100
d = logspace(-3,0,lend); %d = logspace(-4,0,lend);
%d = 0.5; % To plot pdf 2.1e-1

% BER initialization
P1 = 2/64;               % Prob. bit 1
% P1 = [0.5 0.18 0.2];

ber = zeros(length(P1),lend);
ber0 = ber;
ber1 = ber;

load mol_abs.mat
do_pulse;
do_att_noise;

for j = 1:length(P1),   
    yy = 1 - P1(j);     % Prob. of bit 0;
    ber_computation;
    ber(j,:)  = BER;
    ber0(j,:) = Ber0;
    ber1(j,:) = Ber1;
end

%figure
loglog(d,ber0,'-ro',d,ber1,'-b*','linewidth',1.5),
xlabel('Distance (m)'), ylabel('Bit error rate'),
ylim([1e-10 1]),
legend('P(e|X=0)','P(e|X=1)'),
grid on

figure
loglog(d,ber,'-o','linewidth',1.5),
xlabel('Distance (m)'), ylabel('Bit error rate'),
ylim([1e-6 1]),
%legend('P(e|X=0)','P(e|X=1)'),
legend('Average BER'),
grid on

save 
