%% Compute attenuation and noise

% Molecular Absorption Loss
tau = sum(kvt(1:8,:),1); % Absorption coefficient [cm^-1]
% absLossDBM = 10*tau*1e2*log10(exp(1)); % Absorption Loss [dB m^-1]
% absLossDB = (d)'*absLossDBM; % Absorption Loss (dB) 
absLossDB = (d)'*10*tau*1e2*log10(exp(1));
% Spreading Loss

% Previously I used the frequency dependent term for the effective area
% spreadLossDB = 20*log10(4*pi*d')*ones(1,length(f))+ones(1,length(d))'*20*log10(f./c);

% But now I compute this to estimate the effective area of the receiving antenna
[maxx,f0] = max(X(1:NFFT/2+1));
% spreadLossDB = 20*log10(4*pi*d')*ones(1,length(f))+ones(1,length(d))'*20*log10(f(f0)*ones(size(f))./c);
spreadLossDB = 20*log10(4*pi*d'*f/c);%*ones(1,length(f));%+ones(1,length(d))'*20*log10(f(f0)*ones(size(f))./c);

% Total Path Loss
pathLossDB = absLossDB+spreadLossDB;
pathLossDB(:,1) = 0; % Path-loss at 0 Hz is always 0
pathLossDB = pathLossDB.*(pathLossDB>=0); % Path-loss expression only valid in the far field, always positive


%% Signals spectrum
X_ = ones(1,length(d))'*X(1:length(f));

% Power spectral density for X
S_X = 1./fs*L.*(X_(:,1:NFFT/2+1).*conj(X_(:,1:NFFT/2+1)))*tmax/real_t*2;
%%%%%% NOTE THAT I'VE ADDED A NORMALIZATION FACTOR!!! %%%%%%%
% I was doing a mistake before by not taking into account that I simulate
% much longer signals than what the useful signal is, thus, I was computing
% a "diluted" p.s.d.

% Y, received signal
Y = X_.*10.^(-pathLossDB/20);
S_Y = 1./fs*L.*(Y(:,1:NFFT/2+1).*conj(Y(:,1:NFFT/2+1)))*tmax/real_t*2;
%%%%%% NOTE THAT I'VE ADDED A NORMALIZATION FACTOR!!! %%%%%%%
% I was doing a mistake before by not taking into account that I simulate
% much longer signals than what the useful signal is, thus, I was computing
% a "diluted" p.s.d.

% % Enery and Power -> JUST TO VERIFY I'M DOING EVERYTHING CORRECTLY
% Power = sum(S_X(1,:))*f(2)
% Energy_f = Power*real_t
% Energy_t = sum(abs(xn).^2)*t(2)
% Power_t1 = Energy_t/tmax
% Power_t2 = Energy_t/real_t


%% Noise model
% Emissivity of the channel
tau_d = exp(-d'*1e2*tau);
e = 1-tau_d;

% Default background noise at room temperature (effective area term
% included as above)
% NOTE: At the end this is not distance dependent, doesn't make sense
% otherwise!
S_N0 = k*ones(size(e))*296.*(c./(sqrt(4*pi)*f(f0))).^2;

% Additional noise due to the transmission at some frequencies (effective
% area term included as above)
S_N1 = e.*(c./(4*pi*d'*f(f0)*ones(size(f)))).^2.*S_X;
S_N1(:,1) = 0;


%%
% finding the maximum value of the p.s.d of transmitted signal and its
% position in the matrix
[max_val,max_pos] = max(S_X(1,:));

% finding the minimum value of the p.s.d of transmitted signal that is 100
% times less than the maximum value of the p.s.d of transmitted signal and
% its position in the matrix
[cut_min,pos_min] = max(S_X(1,1:max_pos).*(S_X(1,1:max_pos)<max_val/100));

[cut_max,pos_max] = min(S_X(1,max_pos:length(S_X)).*(S_X(1,max_pos:length(S_X))>max_val/100));

pos_max = max_pos+pos_max;

EX = sum(S_X(1,pos_min+1:pos_max),2)*f(2); % X power

EY_vec = sum(S_Y(:,pos_min+1:pos_max),2)*f(2); % Y power

N0_vec = sum(S_N0(:,pos_min+1:pos_max),2)*f(2); % Background noise power
N1_vec = N0_vec+sum(S_N1(:,pos_min+1:pos_max),2)*f(2); % Total noise power

% figure,
% loglog(d,EY_vec*1e9,'-*',d,N1_vec*1e9,'-*',d,N0_vec*1e9,'-*'),
% legend('Power','Noise_1','Noise_0'),
% ylabel('Power [nW]')

%save noise_e18.mat EY_vec N1_vec N0_vec Emax Power d lend

% %% For the capacity in ICC
% snr = S_Y./S_N1;
% cap = 1/(2*max(t))*sum(log2(1+snr(:,cut_min+1:cut_max)),2);
% % 
% % % loglog(d,N1_vec,d,EY_vec)
% % % grid on
% % 
% % % Capacity of ICC
% figure,
% loglog(d,sum(log2(1+snr(:,cut_min+1:cut_max))*f(2),2)),
% title('AWGN chan'), grid

